<?php
	/*
	Plugin Name: Woocommerce PaymentAPI.io Gateway
	Description: This plugin allows you to accept cryptocurrency anonymously on your Woocommerce site
	Version: 1.0.1
	*/

    add_action('admin_notices', function () {
    
    	if (!class_exists('WooCommerce')) {
    		?>
            <div id="woo-warning" class="notice notice-error sf-notice-nux is-dismissible">
    			<?php esc_html_e('In order to use Woocommerce PaymentAPi, You must install WooCommerce plugin', 'paymentapi'); ?>
            </div>
    		<?php
    	}
    
    });

		add_action( 'plugins_loaded', 'init_paymentapi_gateway_class' );
		function init_paymentapi_gateway_class() {
		    if (in_array('woocommerce/woocommerce.php', get_option('active_plugins'))) {
			if ( ! class_exists( 'WC_PaymentAPI_Gateway' ) ) {

			    class WC_PaymentAPI_Gateway extends WC_Payment_Gateway {

			        public function __construct() {
			            $this->id = 'paymentapi'; // Gateway ID
			            $this->icon = ''; // URL to your gateway's icon
			            $this->method_title = 'PaymentAPI.io'; // Gateway title
			            $this->method_description = 'Pay using PaymentAPI.io'; // Gateway description
			            $this->has_fields = false; // Set to true if you need payment fields
			            $this->title = get_option('woocommerce_paymentapi_settings')['subscription_title'];
					    $this->description = get_option('woocommerce_paymentapi_settings')['subscription_description'];
			            // Initialize gateway settings
			            $this->init_paymentapi_form_fields();
			            $this->init_settings();
			            // Hook into WooCommerce actions
			            add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
			        }

			        // Initialize gateway settings fields
			        public function init_paymentapi_form_fields() {
			            // set ipnkey on settings load
			            if(strlen(get_option('wc_paymentapi_ipnkey'))<1){
			                update_option('wc_paymentapi_ipnkey',rand(1111111111,999999999999));
			            }
			            
			            $this->form_fields = array(
					    	'enabled' => array(
					          'title' => __('Enable/Disable', 'woocommerce'),
					          'type' => 'checkbox',
					          'label' => __('Enable PaymentAPI Gatway', 'woocommerce'),
					          'default' => 'yes'
					        ),			        
					        /*
					        'paymentapi_currency' => array(
					          'title' => __('Currency', 'woocommerce'),
					          'type' => 'multiselect',
					          'options' => array(
					          	'btc' => __('BTC', 'woocommerce'),
					          	'ltc' => __('LTC', 'woocommerce'),
					          	'doge' => __('DOGE', 'woocommerce'),
					          ),
					          'required'=>'true'
					        ),*/	
					        'paymentapi_payout_address_btc' => array(
					          'title' => __('Bitcoin Payout Address', 'woocommerce'),
					          'type' => 'text'
					        ),
					        'paymentapi_payout_address_ltc' => array(
					          'title' => __('Litecoin Payout Address', 'woocommerce'),
					          'type' => 'text'
					        ),
					        'paymentapi_payout_address_doge' => array(
					          'title' => __('Dogecoin Payout Address', 'woocommerce'),
					          'type' => 'text'
					        ),
					        /*'paymentapi_ipn_key' => array(
					          'title' => __('IPN Key', 'woocommerce'),
					          'type' => 'text'
					          
					        ),*/
					        'subscription_title' => array(
					          'title' => __("Gateway Name", "woocommerce"),
					          'type' => 'text'
					        ),
					        'subscription_description' => array(
					          'title' => __("Gateway Description", "woocommerce"),
					          'type' => "textarea"
					        )
					    );
			        }
			        
			        
			       

			        // Process payment
			        public function process_payment( $order_id ) {
			            global $woocommerce;
					    $order = new WC_Order( $order_id );
					    
					    $payout_address = get_option('woocommerce_paymentapi_settings')['paymentapi_payout_address_'.$_REQUEST['paymentapi_currency']];
					    $return_url = get_bloginfo('url').'/';
					    $currency = $_REQUEST['paymentapi_currency'];
					    //$ipn_key = get_option('woocommerce_paymentapi_settings')['paymentapi_ipn_key'];
					    $ipn_key =get_option('wc_paymentapi_ipnkey');
					    $amount = $order->get_total();
					    
					    $curl = curl_init();
					    $url_params = http_build_query([
                            'payout_address' => $payout_address,
                            'amount' => $amount,
                            'currency' => $currency,
                            'return_url' => "$return_url?ordersuccess=1&order=$order_id",
                            'ipn_url' => get_bloginfo('url')."/?ipn_key=$ipn_key",
                            'ipn_data' => $order_id,
                            'ipn_key' => $ipn_key
                            ]);
					    $url = "https://gateway.paymentapi.io/api/create_payment?$url_params";
					    curl_setopt($curl, CURLOPT_URL, $url);
						curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
						$output = json_decode(curl_exec($curl));
						curl_close($curl);
						return array(
				        	'result' => 'success',
				        	'redirect' => $output->invoice_url
				    	);
						
			        }
			    }
			}
		    }
			
			// to process ipn from gateway
			add_action ( 'init', 'get_payment_api_ipn_update');
			function get_payment_api_ipn_update(){
			   
			    
			    if(get_option('wc_paymentapi_ipnkey')==$_POST['ipn_key'] && $_POST['confirms'] > 1 ) {
			        if (in_array('woocommerce/woocommerce.php', get_option('active_plugins'))) {
						$order_id = $_POST['ipn_data'];
						$order = new WC_Order($order_id);
						$order->reduce_order_stock();
						
						$get_order_key = $get_order->order_key;
						
						$order->payment_complete();
						$order->update_status('completed');
						$order->add_order_note('IPN Data: '.$_REQUEST['ipn_data']);
			        }
                    
					exit;	
			    }
			}
			
			//redirect to thank you page from gateway
			add_action ( 'init', 'get_payment_api_callback_data');
			function get_payment_api_callback_data(){
			    if($_REQUEST['ordersuccess']==1){
			        $order_id = $_REQUEST['order'];
			       if (in_array('woocommerce/woocommerce.php', get_option('active_plugins'))) {
			    	$order = new WC_Order($order_id);
			    	WC()->cart->empty_cart();
		    		$get_order_key = $order->order_key;
					$url=home_url()."/checkout/order-received/".$order_id."/?key=".$get_order_key;
					wp_redirect($url);
			        }
					exit;
			    }
			}

			// Add the gateway to WooCommerce
			function add_paymentapi_gateway( $gateways ) {
			    $gateways[] = 'WC_PaymentAPI_Gateway';
			    return $gateways;
			}
			add_filter( 'woocommerce_payment_gateways', 'add_paymentapi_gateway' );

		}


add_action( 'wp_enqueue_scripts', 'crypto_plugin_assets' );
function crypto_plugin_assets() {
    wp_enqueue_style( 'checkout-style', plugins_url( '/assets/style.css' , __FILE__ ) );
    wp_enqueue_script( 'checkout-script', plugins_url( '/assets/scripts.js' , __FILE__ ),array('jquery'),false );
}



// add currency option to gateway

add_filter( 'woocommerce_gateway_description', 'gateway_paymentapi_currency', 20, 2 );
function gateway_paymentapi_currency( $description, $payment_id ){
    //
    if( 'paymentapi' === $payment_id ){
        ob_start(); // Start buffering
            
        echo '<div  class="paymentapi-fields" style="padding:10px 0;">';
        $currencies=array('btc','ltc','doge');
        $c = array();
        
        if((is_array($currencies))){
			foreach($currencies as $currency){
				if(get_option('woocommerce_paymentapi_settings')['paymentapi_payout_address_'.$currency]!=''){
					$c[$currency]=strtoupper($currency);
				}
			}
			
			woocommerce_form_field( 'paymentapi_currency', array(
				'type'          => 'select',
				'label'         => __("Select a currency", "woocommerce"),
				'class'         => array('form-row-wide w-100'),
				'required'      => true,
				'options' => $c,
			), '');
        } else {
        echo 'No currency is selected in the settings';
        }

        
        echo '<div>';
    }
        $description .= ob_get_clean(); // Append buffered content
    
    return $description;
}

// Checkout custom field save to order meta
add_action('woocommerce_checkout_create_order', 'gateway_paymentapi_option_order_meta', 10, 2 );
function gateway_paymentapi_option_order_meta( $order, $data ) {
    if ( isset($_POST['paymentapi_currency']) && ! empty($_POST['paymentapi_currency']) ) {
        $order->update_meta_data( '_paymentapi_currency' , esc_attr($_POST['paymentapi_currency']) );
    }
}


// Display custom field in Admin orders, below billing address block
add_action( 'woocommerce_admin_order_data_after_billing_address', 'gateway_paymentapi_display_order_billing_address', 10, 1 );
function gateway_paymentapi_display_order_billing_address( $order ){
    if( $paymentapi_option = $order->get_meta('_paymentapi_currency') ) {
        echo '<div class="paymentapi-option">
        <p><strong>'.__('PaymentApi Currency').':</strong> ' . $paymentapi_option . '</p>
        </div>';
    }
}
	
?>